'use strict';

/**
 * @file
 * Do not let videos be played until accept Onetrust cookies.
 */

/**
 * Manage youtube cookies on the page played videos.
 *
 * @param action
 *   Block youtube videos until accept cookies / use youtube-nocookie domain.
 *
 * @param popupMessage
 *   Message that explains the cookies tracking.
 * @param popupButton
 *   Allow managing cookies.
 * @param okButton
 *   Accept youtube's tracking.
 * @param exitButton
 *   Remove the button (only appears in popup).
 *
 * @constructor
 */
function YoutubeCookies(action, popupMessage, popupButton, okButton, exitButton) {
  this.checkYoutubeNoCookie = action !== 'no_cookies_domain';
  this.youtubeCookiesAccepted = false;
  this.popupButton = popupButton;
  this.popupMessage = popupMessage;
  this.action = action;
  this.showThirdPartyButton = true;
  this.okButton = okButton;
  this.manageCookiesCallback = null;
  this.currentVideo = null;
  this.exitButton = exitButton;
  this.acceptedCookiesCallback = null;
  this.interceptVideo = true;
  this.setup();
}

/**
 * Decide wether the manage cookies button should be shown.
 *
 * @param value
 *   If false, the 'Manage cookies' button won't appear.
 */
YoutubeCookies.prototype.setShowThirdPartyButton = function (value) {
  this.showThirdPartyButton = value;
};

/**
 * Callback that allow third parties show the manage cookies form.
 *
 * @param callback
 *   Callback that show the form.
 */
YoutubeCookies.prototype.setManageCookiesCallback = function (callback) {
  this.manageCookiesCallback = callback;
};

/**
 * Check that cookies are accepted.
 *
 * It allows to use a custom callback to determine that.
 *
 * @returns {boolean|*}
 *   TRUE when the cookies are accepted.
 */
YoutubeCookies.prototype.areCookiesAccepted = function () {
  if (this.acceptedCookiesCallback !== null) {
    return this.acceptedCookiesCallback();
  }
  else {
    return this.youtubeCookiesAccepted;
  }
}

/**
 * Setup the youtube videos.
 *
 * Show / hide videos whether whether the
 * cookies have been accepted.
 */
YoutubeCookies.prototype.setAcceptedCookies = function(youtubeCookiesAccepted) {
  if (youtubeCookiesAccepted && !this.youtubeCookiesAccepted) {
    this.setOriginalOembedButtons();
    this.setYoutubeOriginalUrls();
  }
  this.youtubeCookiesAccepted = youtubeCookiesAccepted;
};

/**
 * Set the original oembed buttons.
 */
YoutubeCookies.prototype.setOriginalOembedButtons = function() {
  document.querySelectorAll('[data-yc-fake-button]').forEach(function (element) {
    element.remove();
  });

  document.querySelectorAll('[data-yc-oembed-button]').forEach(function (element) {
    element.style.display = '';
  });
};

/**
 * Set the callback that will be called when the cookies are not accepted.
 *
 * @param callback
 */
YoutubeCookies.prototype.onNonAcceptedCookies = function (callback) {
  this.onNonAcceptedCookiesCallback = callback;
};

/**
 * Setup the cookies management.
 */
YoutubeCookies.prototype.setup = function () {
  document.addEventListener('DOMContentLoaded', this.onContentLoaded.bind(this));
};

/**
 * Adds the youtube cookies popup.
 *
 * @param element
 *   Element that will have the popup.
 * @param showExitButton
 *   Weather to show the exit button or not.
 *
 * @returns {HTMLDivElement}
 */
YoutubeCookies.prototype.addPopup = function(element, showExitButton) {
  var popupContainer = document.createElement('div');
  popupContainer.classList.add('youtube-cookies-popup');
  var popupContainerBox = document.createElement('div');
  popupContainerBox.classList.add('youtube-cookies-popup-box');
  var popupContainerBoxInfo = document.createElement('div');
  popupContainerBoxInfo.classList.add('youtube-cookies-popup-box-info');
  popupContainerBoxInfo.insertAdjacentHTML('beforeend', this.popupMessage);
  var popupContainerBoxButtons = document.createElement('div');
  popupContainerBoxButtons.classList.add('youtube-cookies-popup-box-buttons');
  if (this.showThirdPartyButton) {
    popupContainerBoxButtons.insertAdjacentHTML('beforeend', this.popupButton);
  }
  popupContainerBoxButtons.insertAdjacentHTML('beforeend', this.okButton);
  if (showExitButton) {
    popupContainerBoxButtons.insertAdjacentHTML('beforeend', this.exitButton);
  }
  popupContainerBoxInfo.appendChild(popupContainerBoxButtons);
  popupContainerBox.appendChild(popupContainerBoxInfo);
  popupContainer.appendChild(popupContainerBox);
  element.append(popupContainer);
  popupContainer.querySelector('button.youtube-cookies-accept').addEventListener('click', this.onAcceptClick.bind(this));
  if (showExitButton) {
    popupContainer.querySelector('button.youtube-cookies-reject').addEventListener('click', this.onRejectClick.bind(this));
  }
  var manageCookiesButton = popupContainer.querySelector('button.youtube-cookies-manage');
  if (manageCookiesButton != null) {
    manageCookiesButton.addEventListener('click', this.onManageCookiesClick.bind(this));
  }
  return popupContainer;
};

/**
 * Actions to do when the fake thumbnail is clicked.
 */
YoutubeCookies.prototype.onThumbnailClick = function (event) {
  if (!this.interceptVideo) {
    return;
  }

  if (!this.areCookiesAccepted()) {
    event.preventDefault();
    this.addPopup(document.body, true);
    this.currentVideo = event.currentTarget.parentNode;
    this.interceptVideo = false;
  }
  else {
    var iframeWrapper = event.currentTarget.parentNode;
    this.removeFakeThumbnailAndPlay(iframeWrapper);
    this.interceptVideo = true;
    this.currentVideo = null;
  }
};

YoutubeCookies.prototype.removeFakeThumbnailAndPlay = function (iframeWrapper) {
  // Hide fake thumbnail and play original one.
  iframeWrapper.querySelector('[data-yc-fake-button]').remove();
  iframeWrapper.querySelector('[data-yc-oembed-button]').style.display = '';
  // Only autoplay the video back when the user accept viewing the video.
  iframeWrapper.querySelector('[data-yc-oembed-button]').click();
};

/**
 * Dispatch event to allow other modules to add
 * their specific behaviours when the thumbnail
 * button is clicked.
 */
YoutubeCookies.prototype.eventDispatch = function(button) {
  var iframeWrapper = button.currentTarget.parentNode;
  const event = new Event('youtube-cookies-thumbnail-click');
  iframeWrapper.querySelector('iframe').dispatchEvent(event);
};

/**
 * Remove the popup.
 */
YoutubeCookies.prototype.removePopup = function(button) {
  if (button instanceof HTMLElement) {
    var popup = button;
    do {
      popup = popup.parentNode;
    } while (!popup.classList.contains('youtube-cookies-popup'))
  }

  if (popup !== null) {
    popup.remove();
  }
};

/**
 * When manage cookies is clicked show the manage cookies button.
 */
YoutubeCookies.prototype.onManageCookiesClick = function (event) {
  if (this.manageCookiesCallback !== null) {
    this.manageCookiesCallback();
  }
  this.interceptVideo = true;
};

/**
 * When cookies are accepted videos can be played.
 *
 * It only works in the current page , after refreshing
 * consent will be needed again. Consent will only stop
 * after managing third party cookies.
 */
YoutubeCookies.prototype.onAcceptClick = function(event) {
  this.removePopup(event.target);

  if (this.currentVideo !== null) {
    this.removeFakeThumbnailAndPlay(this.currentVideo);
    var oembed_button = this.currentVideo.querySelector('[data-yc-oembed-button]');
    if (oembed_button !== null) {
      oembed_button.click();
    }
    this.interceptVideo = true;
    this.currentVideo = null;
  }
};

/**
 * When cookies are rejected the popup is removed.
 */
YoutubeCookies.prototype.onRejectClick = function(event) {
  this.removePopup(event.target);
  this.currentVideo = null;
  this.interceptVideo = true;
};

/**
 * Replace a youtube iframe with a youtube thumbnail and a popup.
 *
 * If tracking is accepted the video will be loaded.
 *
 * @param iframe
 *   Iframe.
 * @param iframeIndex
 *   Used to identify the iframe after cookie tracking is accepted.
 */
YoutubeCookies.prototype.setYoutubeThumbnail = function (iframe, iframeIndex) {
  // @TODO: set youtube thumbnail when no cookies are accepted and
  // method is popup
  var videoid = this.getYoutubeIdFromUrl(iframe.src);
  var thumbnailMarkup = '<source srcset="//i.ytimg.com/vi_webp/' + videoid + '/sddefault.webp, //i.ytimg.com/vi_webp/' + videoid + '/maxresdefault.webp 2x" type="image/webp"><source srcset="//i.ytimg.com/vi/' + videoid + '/sddefault.jpg, //i.ytimg.com/vi/' + videoid + '/maxresdefault.jpg 2x" type="image/jpeg"><img class="oembed-lazyload__img" src="//i.ytimg.com/vi/' + videoid + '/sddefault.jpg" width="300" height="182">';
  var picture = document.createElement('picture');
  picture.setAttribute('data-yc-iframe-index', iframeIndex);
  picture.classList.add('youtube-cookies-iframe-thumbnail');
  var popup = this.addPopup(picture, false);
  popup.querySelector('button.youtube-cookies-accept').setAttribute('data-yc-iframe-index', iframeIndex);
  popup.querySelector('button.youtube-cookies-accept').addEventListener('click', function (event) {
    var index = this.getAttribute('data-yc-iframe-index');
    var iframe = document.querySelector('iframe[data-yc-iframe-index="' + index + '"]');
    iframe.setAttribute('src', iframe.getAttribute('data-src'));
    iframe.addEventListener('load', function (event) {
      event.currentTarget.contentWindow.postMessage('{"event":"command","func":"playVideo","args":""}', '*');
    });

    document.querySelector('picture[data-yc-iframe-index="' + index + '"]').remove();
  });

  picture.insertAdjacentHTML('beforeend', thumbnailMarkup);
  iframe.parentNode.prepend(picture);
  iframe.setAttribute('data-src', iframe.src);
  iframe.src = '';
};

/**
 * Check that a url is from youtube.
 *
 * @param source
 *   Youtube url.
 *
 * @returns {boolean}
 *   True if URL is from youtube.
 */
YoutubeCookies.prototype.isYoutubeSource = function (source) {
  var regExps = [
    /youtu\.be/,
    /youtube\.com/,
  ];

  if (this.checkYoutubeNoCookie) {
    regExps.push(/youtube-nocookie\.com/);
  }

  for (var i in regExps) {
    if (source.match(regExps[i])) {
      return true;
    }
  }
  return false;
};

/**
 * Stop an iframe.
 *
 * @param iframeIndex
 *   Internal identifier of the iframe.
 */
YoutubeCookies.prototype.stopIframe = function(iframeIndex) {
  if (window.navigator.appName === 'Microsoft Internet Explorer') {
    window.frames[iframeIndex].document.execCommand('Stop');
  } else {
    window.frames[iframeIndex].stop();
  }
};

/**
 * Given a youtube video URL get its ID.
 *
 * @param url
 *   Youtube video URL.
 *
 * @returns {null|*}
 *   Video id, if exists.
 */
YoutubeCookies.prototype.getYoutubeIdFromUrl = function(url) {
  const youtubeIdRegexp = '[^\\s&\?\/\#]{11}';
  var videoIdMatches = [];
  if (url.match('(youtube\.com\/embed\/)(' + youtubeIdRegexp + ')')) {
    videoIdMatches = url.match('(youtube\.com\/embed\/)(' + youtubeIdRegexp + ')');
  }
  if (url.match('(youtu\.be\/)(' + youtubeIdRegexp + ')')) {
    videoIdMatches = url.match('(youtube\.com\/embed)|(youtu\.be\/)(' + youtubeIdRegexp + ')');
  }
  else if (url.match('(youtube\.com\/watch%3Fv%3D)(' + youtubeIdRegexp + ')')) {
    videoIdMatches = url.match('(youtube\.com\/watch%3Fv%3D)(' + youtubeIdRegexp + ')');
  }
  if (videoIdMatches !== null && videoIdMatches.length > 0) {
    return videoIdMatches[videoIdMatches.length - 1];
  }
  return null;
};

/**
 * Replace a element youtube URL by youtube-nocookie alternative.
 *
 * @param element
 *   Element.
 * @param url
 *   Video URL.
 */
YoutubeCookies.prototype.setYoutubeNocookiesSrc = function (element, url) {
  var attribute = typeof(element.src) === 'string' && element.src.length > 0 ? 'src' : 'data-src';
  var video_id = this.getYoutubeIdFromUrl(url);
  if (video_id != null) {
    element.setAttribute('data-original-src', url);
    element.setAttribute('data-original-src-attribute', attribute);
    element.setAttribute(attribute, 'https://www.youtube-nocookie.com/embed/' + video_id + '?enablejsapi=1');
  }
};

/**
 * Set the youtube original urls to the iframes.
 *
 * Only works with videos that haven't been played yet.
 */
YoutubeCookies.prototype.setYoutubeOriginalUrls = function() {
  var videos = document.querySelectorAll('iframe[data-original-src*="youtube.com"],iframe[data-original-src*="youtube.com"],iframe[data-original-src*="youtu.be"],iframe[data-original-src*="youtu.be"]');
  for (var video in Array.from(videos)) {
    var attribute = videos[video].getAttribute('data-original-src-attribute');
    videos[video].setAttribute(attribute, videos[video].getAttribute('data-original-src'));
  }
};

/**
 * Setup the iframes so cookies can be managed.
 *
 * @param event
 */
YoutubeCookies.prototype.onContentLoaded = function (event) {
  if (!this.youtubeCookiesAccepted) {
    this.processYoutubeIframes(event.target);
  }
};

/**
 * Process youtube iframes.
 *
 * @param container
 */
YoutubeCookies.prototype.processYoutubeIframes = function (container) {
  // Force attachment of oembed lazyload behaviour before removing the oembed attributes.
  if (typeof(Drupal.behaviors.oembedLazyload) === 'object' && typeof(Drupal.behaviors.oembedLazyload.attach) === 'function') {
    Drupal.behaviors.oembedLazyload.attach(container);
  }
  const iframes = container.getElementsByTagName('iframe');
  for (var iframeIndex in iframes) {
    var iframe = iframes[iframeIndex];
    if (typeof(iframe) !== 'object') {
      return;
    }

    if (iframe.hasAttribute('data-yc-iframe-processed')) {
      continue;
    }

    iframe.parentNode.removeAttribute('data-strategy');
    if (typeof(iframe.src) === 'string'  && iframe.src.length > 0 && this.isYoutubeSource(iframe.src)) {
      switch (this.action) {

        case 'no_cookies_domain':
          this.stopIframe(iframeIndex);
          this.setYoutubeNocookiesSrc(iframe, iframe.src);
          break;

        case 'popup':
          this.stopIframe(iframeIndex);
          this.setYoutubeThumbnail(iframe, iframeIndex);
          iframe.setAttribute('data-yc-iframe-index', iframeIndex);
          break;

      }
    }

    if (iframe.hasAttribute('data-src') && this.isYoutubeSource(iframe.getAttribute('data-src'))) {
      switch (this.action) {

        case 'no_cookies_domain':
          this.setYoutubeNocookiesSrc(iframe, iframe.getAttribute('data-src'));
          break;

        case 'popup':
          iframe.setAttribute('data-yc-iframe-index', iframeIndex);
          if (iframe.parentNode.classList.contains('oembed-lazyload--youtube')) {
            var button = iframe.parentNode.querySelector('button');
            if (typeof(button) === 'object') {
              iframe.parentNode.insertAdjacentHTML('afterbegin', button.outerHTML);
              button.setAttribute('data-yc-oembed-button', 1);
              button.style.display = 'none';
              var newButton = iframe.parentNode.querySelector('button:not([data-yc-oembed-button])');
              newButton.addEventListener('click', this.onThumbnailClick.bind(this));
              button.addEventListener('click', this.eventDispatch.bind(this));
              newButton.setAttribute('data-yc-fake-button', 1);
            }
          }
          break;

      }

    }
    iframe.setAttribute('data-yc-iframe-processed', 1);
  }
};

/**
 * Set the callback that checks cookies have been accepted.
 *
 * @param callback
 *   Callback that determine if cookies have been accepted.
 */
YoutubeCookies.prototype.setAcceptedCookiesCallback = function (callback) {
  this.acceptedCookiesCallback = callback;
};

/**
 * Removes the popup when the YouTube cookie category has been accepted.
 */
YoutubeCookies.prototype.removePopupOnAcceptedCategory = function () {
  if (Drupal.youtubeCookies.areCookiesAccepted()) {
    var popup = document.querySelector(".youtube-cookies-popup");
    if (popup !== null) {
      popup.remove();
    }
  }
};

/**
 * Initialize youtube cookies.
 */
(function (Drupal, drupalSettings) {
  if (typeof(drupalSettings.youtubeCookies) === 'object') {
    Drupal.youtubeCookies = new YoutubeCookies(drupalSettings.youtubeCookies.action, drupalSettings.youtubeCookies.popupMessage, drupalSettings.youtubeCookies.popupButton, drupalSettings.youtubeCookies.okButton, drupalSettings.youtubeCookies.exitButton);
  }

  Drupal.behaviors.youtubeCookies = {
    attach: function (context) {
      Drupal.youtubeCookies.processYoutubeIframes(document);
    }
  };

} (Drupal, drupalSettings));
